#!/usr/bin/perl -w
use strict;
use POSIX;
use Text::ParseWords;

START:

#FIXME needs to understand the jobmanager type blah vs lcgpbs and set it up

my $sigset = POSIX::SigSet ->new();
my $actionHUP=POSIX::SigAction->new("sigHUP_handler",$sigset,&POSIX::SA_NODEFER);
my $actionInt=POSIX::SigAction->new("sigINT_handler",$sigset,&POSIX::SA_NODEFER);
POSIX::sigaction(&POSIX::SIGHUP, $actionHUP);
POSIX::sigaction(&POSIX::SIGINT, $actionInt);
POSIX::sigaction(&POSIX::SIGTERM, $actionInt);
my $configFilePath ="/opt/glite/etc/glite-dgas-ce-pushd.conf"; 
my %configValues = (
		    dgasURDir         =>  "/opt/glite/var/dgasURBox",
		    dgasErrDir        =>  "/opt/glite/var/dgasURBox/ERR/",
		    qDepth 	      => "10",
		    qMult	      => "3",
		    lockFileName      => "/opt/glite/var/glite-dgas-ce-pushd.lock",
		    mainPollInterval  => "5",
		    queuePollInterval =>"25",
		    defaultUserHLR    =>"",
		    forceLocalFirst   =>"no",
		    forceLocalOnly    =>"no",
		    gridUser	      =>"dgas",
		    );

#Parse configuration file
if(exists $ARGV[0]) 
{
    $configFilePath = $ARGV[0];
}	
&parseConf($configFilePath); 

## Global variable initialization
my %hash;
my $UR;
my $list;
my @queue = ();

my $qDept = $configValues{qDepth};
my $qMult = $configValues{qMult}; 

my $dgas_dir     = $configValues{dgasURDir};
my $dgas_err_dir = $configValues{dgasErrDir};

my $lockFileName = $configValues{lockFileName};

my $T2 = $configValues{queuePollInterval};
my $T1 = $configValues{mainPollInterval};

my $defaultUserHLR =$configValues{defaultUserHLR};

my $forceLocalFirst =$configValues{forceLocalFirst};
my $forceLocalOnly =$configValues{forceLocalOnly};

my @dvec = split(/\//, $dgas_err_dir);

my $errDirName =  $dvec[$#dvec];
my $keepGoing = 1;
my $start = time;   ##  time init

my $vomsProxyInfo = "";
#put lock
if ( &putLock($lockFileName) != 0 )
{
    die "Fatal Error: Couldn't open lock file! in $lockFileName\n";
}
else
{
	print "Lock file succesfully created.\n";
}

( -d $configValues{dgasURDir} ) || mkdir $configValues{dgasURDir};

#create dir of errors
&createErrDir();

if ( &findVomsProxyInfo($vomsProxyInfo) == 0 )
{
        print "Found voms-proxy-info: $vomsProxyInfo\n";
}
else
{
        print "Could not find voms-proxy-info\n";
}
print "Notice: Found voms-proxy-info: $vomsProxyInfo\n";


while( $keepGoing )
{
    
##----------------------------------------------------------------  STEP1
    opendir(DIR, $dgas_dir) || die "can't opendir $dgas_dir: $!";
    while( defined(my $file = readdir(DIR)) && $keepGoing ) 
    {
	next if ($file =~ /^\.\.?$/ || $file =~ /.proxy$/ || $file =~ /^$errDirName$/);	# skip . .. proxy files
	if( &checkQueue($file) ==0 )           ## if file is not .* or .proxy
	{  
	    &parseFile($file);         ## if file is a true input file   
	    #  &printOut;
	    if(&callAtmClient($file) == 0){&delFile($file);}
	    else{&pushToQueue($file);}
	}
    }
    closedir DIR; 
    if ( $keepGoing )
    {
    	sleep($T1);
    }

##----------------------------------------------------------------  END STEP1 
    
    if((&timeOut() == 0) && ($#queue != -1) && $keepGoing )    ##timeout reached
    {
	my @localQueue = ();
	
	@queue = sort { ( ($a =~ /:(\d*):\d*$/)[0] <=> ($b =~ /:(\d*):\d*$/)[0] )
			    || 
			    ( ($a =~ /:(\d*)$/)[0] <=> ($b =~ /:(\d*)$/)[0] ) } @queue;
	
	foreach(@queue)
	{
	    my @vec = split(/:/, $_);
	    
	    
	    if(-e $dgas_dir.$vec[0])
	    {
		&parseFile($vec[0]);
		if(&callAtmClient($vec[0]) == 0)
		{
		    &delFile($vec[0]);
		}
		elsif ($vec[1]>= $qMult)
		{
		    &moveInErrDir($vec[0]);
		}
		else
		{
		    my $updatedItem = &setFile(@vec);
		    push @localQueue , $updatedItem;
		}
	    }
	}
	
	@queue =  @localQueue;  #  update queue 
	splice @localQueue, 0;
    }
    
}   

&Exit();

sub Exit ()
{
	print "Exiting...\n";
	if ( &delLock($lockFileName) != 0 )
	{	
	    print "Error removing lock file.\n";
	}
	else
	{
		print "Lock file removed.\n";
	}
	print "Exit.\n";
}

###-----------------------END-----------------------------------------###



sub parseConf 
{
    my $fconf = $_[0];
    open(FILE, "$fconf") || die "file not open \n";
    while(<FILE>)
    {
	if (/\$\{(.*)\}/)
        {
                my $value=$ENV{$1};
                s/\$\{$1\}/$value/g;
        }
	if(/^qDept\s*=\s*\"(.*)\"$/){$configValues{qDept}=$1;}
	if(/^qMult\s*=\s*\"(.*)\"$/){$configValues{qMult}=$1;} 
	if(/^dgasURDir\s*=\s*\"(.*)\"$/){$configValues{dgasURDir}=$1;}
	if(/^dgasErrDir\s*=\s*\"(.*)\"$/){$configValues{dgasErrDir}=$1;}
	if(/^lockFileName\s*=\s*\"(.*)\"$/){$configValues{lockFileName}=$1;}
	if(/^mainPollInterval\s*=\s*\"(.*)\"$/){$configValues{mainPollInterval}=$1;}
	if(/^queuePollInterval\s*=\s*\"(.*)\"$/){$configValues{queuePollInterval}=$1;}
	if(/^lrmsType\s*=\s*\"(.*)\"$/){$configValues{lrmsType}=$1;}
	if(/^defaultUserHLR\s*=\s*\"(.*)\"$/){$configValues{defaultUserHLR}=$1;}
	if(/^forceLocalFirst\s*=\s*\"(.*)\"$/){$configValues{forceLocalFirst}=$1;}
	if(/^forceLocalOnly\s*=\s*\"(.*)\"$/){$configValues{forceLocalOnly}=$1;}
	if(/^gridUser\s*=\s*\"(.*)\"$/){$configValues{gridUser}=$1;}
    }
    close(FILE);
}


sub checkQueue
{
    foreach (@queue)
    {
	my $file = $_;
	if($file =~ /($_[0]:\d*:\d*)/){ return 1;}   ## file in queue
    }
    
    return 0;
}      


sub pushToQueue 
{
    my $file= "$_[0]:0:0";
    unshift @queue, $file; 
    
    return 0;
}



sub parseFile
{
	my $file = $_[0];
	my $queueName = "";
	if ( $file =~ "pbs_" )
	{
		#This is a UR file for pbs LRMS
		return &parseFile_PBS($file);
		
	}
	if ( $file =~ "lsf_" )
	{
		#This is a UR file for lsf LRMS
		return &parseFile_LSF($file);
	}
	#unknown file type
	return 1;
}

sub parseFile_PBS 
{
    $UR   = "";
    %hash = (
	     server   =>  "server",
	     queue    =>  "queue",
	     lrmsType =>  "pbs",
	     jobmanagerType =>  "blah",
             globusResContact =>  "GLOBUS_RES_CONTACT",
	     cput     =>  "cput",
	     hlrLoc   =>  "HLR_LOCATION",
	     jobId    =>  "WMS_JOBID",
	     jobIdEdg =>  "EDG_WL_JOBID",
	     user     =>  "PBS_O_LOGNAME",
	     mem      =>  "mem",
	     vmem     =>  "vmem", 
	     walltime =>  "walltime",		   
	     lrmsId   =>  "lrmsId",		   
	     jobName  =>  "jobName",		  
	     group    =>  "group", 
	     ctime    =>  "ctime", 
	     qtime    =>  "qtime", 
	     etime    =>  "etime", 
	     start    =>  "start", 
	     end      =>  "end", 
	     exitStatus      =>  "exitStatus", 
	     execHost    =>  "execHost", 
	     processors => "processors", 
	     si2k => "GlueHostBenchmarkSI00", 
	     sf2k => "GlueHostBenchmarkSF00", 
	     tz => "timeZone", 
	     );    
    
    open(FILE, "< $dgas_dir/$_[0]") || return 1;
    
    while(<FILE>)
    {
	if (/^$hash{jobId}=(.*)$/) #get GLITE_WMS_JOBID
	{
	#job coming from a gLite RB 
		$hash{jobId}=$1;
	}
	if ( $hash{jobId} ne "WMS_JOBID" )
	{
                my $var="ACCTLOG:";
                if (/^$var(.*)$/){$UR=$1;}#get the UR string
	}
	if (/^$hash{jobIdEdg}=(.*)$/)#get EDG_WL_JOBID
	{
	#job coming from a edg-like RB
		$hash{jobIdEdg}=$1;
	}
	if ( $hash{jobIdEdg} ne "EDG_WL_JOBID" )
	{
		my $var="ACCTLOG:";
		if (/^$var(.*)$/){$UR=$1;}#get the UR string
	}
	if (/^$hash{hlrLoc}=(.*)$/){$hash{hlrLoc}=$1};#get HLR_LOCATION
	if (/^$hash{globusResContact}=(.*)$/){$hash{globusResContact}=$1};#get GLOBUS_RES_CONTACT
	if (/^$hash{si2k}=(.*)$/){$hash{si2k}=$1};
	if (/^$hash{sf2k}=(.*)$/){$hash{sf2k}=$1};
	if (/^$hash{tz}=(.*)$/){$hash{tz}=$1};
    }
    close(FILE);
    if ( $hash{jobId} ne "WMS_JOBID" )
    {
	#gLite	
    	if ( $UR ne "" )
    	{
		print "parseUR_PBS\n";
 		&parseUR_PBS($UR);
    	}
    }
    if ( $hash{jobIdEdg} ne "EDG_WL_JOBID" )
    {
	#edg-like
	if ( $UR ne "" )
	{
		&parseUR_PBS($UR);
	}
    }
    return 0;
}

sub parseFile_LSF
{
	#FIXME
	$UR   = "";
	%hash = (
		server   =>  "server",
		queue    =>  "queue",
	        lrmsType =>  "lsf",
	     	jobmanagerType =>  "blah",
		globusResContact =>  "GLOBUS_RES_CONTACT",
	    	hlrLoc   =>  "HLR_LOCATION",
	     	jobId    =>  "WMS_JOBID",
	     	user     =>  "PBS_O_LOGNAME",
	     	mem      =>  "mem",
	     	vmem     =>  "vmem", 
		cput     =>  "cput",
	     	walltime =>  "walltime",		   
	     	lrmsId   =>  "lrmsId",		  
	        jobName  =>  "jobName",		  
   	        group    =>  "group", 
	        ctime    =>  "ctime", 
	        qtime    =>  "qtime", 
	        etime    =>  "etime", 
	        start    =>  "start", 
	        end      =>  "end", 
	        exitStatus      =>  "exitStatus", 
	        execHost    =>  "execHost", 
		processors => "processors", 
	        si2k => "GlueHostBenchmarkSI00", 
	        sf2k => "GlueHostBenchmarkSF00", 
	        tz => "timeZone", 
	     );    
	open(FILE, "< $dgas_dir/$_[0]") || return 1;
	while(<FILE>)#FIXME NEW FILE FORMAT
	{
		if (/^$hash{hlrLoc}=(.*)$/){$hash{hlrLoc}=$1};#get HLR_LOCATION
		if (/^$hash{jobId}=(.*)$/){$hash{jobId}=$1};#get EDG_WL_JOBID
		if (/^$hash{globusResContact}=(.*)$/){$hash{globusResContact}=$1};#get GLOBUS_RES_CONTACT
		if (/^$hash{si2k}=(.*)$/){$hash{si2k}=$1};
		if (/^$hash{sf2k}=(.*)$/){$hash{sf2k}=$1};
		if (/^$hash{tz}=(.*)$/){$hash{tz}=$1};
		if (/^ACCTLOG:(.*)$/){$UR=$1};#get the UR string
	}
	close(FILE);
	if ( $UR ne "" )
	{
		&parseUR_LSF($UR);
	}
	return 0;
}

sub parseUR_PBS
{
	my $URString = $_[0];
	print "Got UR string:$URString\n";
	my @URArray = split ( ' ' , $URString );
	my @tmpArray = split ( ';', $URArray[1] );
	$_ = $tmpArray[3];
	if (/^user=(.*)$/){$hash{user}=$1;};
	$hash{lrmsId}=$tmpArray[2];
	$_ = $tmpArray[2];
	if (/^(\d*)\.(.*)$/){$hash{server}=$2;};
	foreach ( @URArray )
	{
		if ( /^queue=(.*)$/){$hash{queue}=$1;};
		if ( /^resources_used.cput=(.*)$/)
		{
			$_=$1;
			$_=~ /(\d\d):(\d\d):(\d\d)$/;
			$hash{cput}= $3 + $2*60 + $1*3600;
		}
		if ( /^resources_used.walltime=(.*)$/)
		{
			$_=$1;
			$_=~ /(\d\d):(\d\d):(\d\d)$/;
			$hash{walltime}= $3 + $2*60 + $1*3600;
		}
		if ( /^resources_used.vmem=(.*)$/)
		{
			$_=$1;
                        $_=~ /(\d*[M.k]b)$/;
			$hash{vmem}     = $1;
		}
		if ( /^resources_used.mem=(.*)$/)
		{
			$_=$1;
			$_=~ /(\d*[M.k]b)$/;
			$hash{mem}     = $1;
		}
		if ( /^Resource_List.neednodes=(.*)$/)
		{	
			$hash{processors} = $1;
		}
		if ( /^group=(.*)$/)
		{	
			$hash{group} = $1;
		}
		if ( /^jobname=(.*)$/)
		{	
			$hash{jobName} = $1;
		}
		if ( /^ctime=(.*)$/)
		{	
			$hash{ctime} = $1;
		}
		if ( /^qtime=(.*)$/)
		{	
			$hash{qtime} = $1;
		}
		if ( /^etime=(.*)$/)
		{	
			$hash{etime} = $1;
		}
		if ( /^start=(.*)$/)
		{	
			$hash{start} = $1;
		}
		if ( /^end=(.*)$/)
		{	
			$hash{end} = $1;
		}
		if ( /^exec_host=(.*)$/)
		{	
			$hash{execHost} = $1;
		}
		if ( /^Exit_status=(.*)$/)
		{	
			$hash{exitStatus} = $1;
		}
	}
	
}

sub parseUR_LSF
{
        my $URString = $_[0];
        my @new = quotewords(" ", 0, $URString);
        my $shift1 = $new[22];
        my $shift2 = $new[23+$shift1];
        $hash{server}=$new[16];
        $hash{queue}=$new[12];
        $hash{user}=$new[11];
        $hash{lrmsId}=$new[3];
        $hash{processors}=$new[6];
        $hash{walltime}=$new[2]-$new[10];
        $hash{cput}=int($new[28+$shift2])+int($new[29+$shift2]);
        $hash{mem}=$new[54+$shift2]."k";
        $hash{vmem}=$new[55+$shift2]."k";
        $hash{start}=$new[10];
        $hash{end}=$new[2];
        $hash{jobName}=$new[26+$shift2];
        $hash{exitStatus}=$new[49+$shift2];
}

sub callAtmClient 
{
    my $now_string = localtime;
    if ( $keepGoing == 0 )
    {
	print "$now_string: Got termination signal..\n";
	return -2;
    }
    my $cmd = "";
    my $status = -1;
    my $resGridId= "";
    my $forceThisJobLocal = "no";
    if ( $hash{globusResContact} eq "GLOBUS_RES_CONTACT" )
    {
       $resGridId="$hash{server}:2119/$hash{jobmanagerType}-$hash{lrmsType}-$hash{queue}";
    }
    else
    {
       $resGridId = $hash{globusResContact};
    }
    if ( $resGridId =~ /-$hash{queue}/ )
    {
        print "$now_string: Found ceId:$resGridId\n";
    }
    else
    {
        $resGridId = $resGridId."-$hash{queue}";
        print "$now_string: Added queue info to ceId:$resGridId\n";
    }
    
    if ( $hash{hlrLoc} eq "HLR_LOCATION" )
    {
        if ($defaultUserHLR ne "" )
        {
		print "$now_string:Using default HLR Location for users: $defaultUserHLR\n";
                $hash{hlrLoc} = $defaultUserHLR;
        }
	else
	{
		print "$now_string:user didn't specify HLR_LOCATION and defaultHlrLocation is empty. Storing URs to resource HLR _only_!\n";
		$forceThisJobLocal = "yes";
	}
    }
    $cmd = "/opt/glite/libexec/glite_dgas_atmClient";
    if ( $hash{jobId} eq "WMS_JOBID" )
    {
        $cmd .= " --jobid $hash{jobIdEdg}";
    }
    else
    {
        $cmd .= " --jobid $hash{jobId}";
    }
    $cmd .= " --resgridid $resGridId";
    $cmd .= " --remhlrid \"$hash{hlrLoc}\"";
    $cmd .= " CPU_TIME=$hash{cput} WALL_TIME=$hash{walltime} PMEM=$hash{mem} VMEM=$hash{vmem}";
    $cmd .= " \"QUEUE=$hash{queue}\" \"USER=$hash{user}\" \"LRMSID=$hash{lrmsId}\"";
    $cmd .= " \"PROCESSORS=$hash{processors}\" \"URCREATION=$now_string\"";
    $cmd .= " \"group=$hash{group}\" \"jobName=$hash{jobName}\"";
    $cmd .= " \"start=$hash{start}\" \"end=$hash{end}\"";
    $cmd .= " \"ctime=$hash{ctime}\" \"qtime=$hash{qtime}\"";
    $cmd .= " \"etime=$hash{etime}\" \"exitStatus=$hash{exitStatus}\"";
    $cmd .= " \"si2k=$hash{si2k}\" \"sf2k=$hash{sf2k}\"";
    $cmd .= " \"tz=$hash{tz}\"";
    my $userDN = "";
    my $fqan = "";
    if ( &findUserFqan("$dgas_dir/$_[0].proxy",$fqan) == 0 )
    {
        print "$now_string: found user Proxy FQAN=$fqan\n";
        $cmd .= " \"fqan=$fqan\"";
    }
    &checkProxy("$dgas_dir/$_[0].proxy", $userDN);
    if ( ( $forceLocalOnly eq "yes" ) || ($forceThisJobLocal eq "yes") )
    {
	print "$now_string: forceLocalOnly is set to \"yes\" in conf file. Therefore UR is sent to resource(site) HLR _only_.\n";
	$cmd .= " --usrcert \"$userDN\"";
	$cmd .= " -3";	
	$status = system("$cmd");
	print "$now_string: Running: $cmd\n";
	if ($? == -1) {
                       print "failed to execute: $!\n";
                   }
                   elsif ($? & 127) {
                       $status = ($? & 127);
                   }
                   else {
                       $status = $? >> 8;
                   }
	print "EXITSTATUS=$status\n\n";
	return $status;
    }	
    elsif ( $forceLocalFirst eq "yes" )
    {
	print "$now_string: forceLocalFirst is set to \"yes\" in conf file. Therefore UR is sent to resource(site) HLR first.\n";
	$cmd .= " --usrcert \"$userDN\"";
	$cmd .= " -2";
	$status = system("$cmd");
    	print "$now_string: Running: $cmd\n";
	if ($? == -1) {
                       print "failed to execute: $!\n";
                   }
                   elsif ($? & 127) {
                       $status = ($? & 127);
                   }
                   else {
                       $status = $? >> 8;
                   }
	print "EXITSTATUS=$status\n";
	return $status;
    }
    elsif ( &checkProxy("$dgas_dir/$_[0].proxy", $userDN) != 0 )
    {
	print "$now_string: proxy $dgas_dir/$_[0].proxy, for job $hash{lrmsId} with griJobId $hash{jobId} is invalid. trying host proxy.\n";
	$cmd .= " --usrcert \"$userDN\"";
	$cmd .= " -2";
	$status = system("$cmd");
    	print "$now_string: Running: $cmd\n";
	if ($? == -1) {
                       print "failed to execute: $!\n";
                   }
                   elsif ($? & 127) {
                       $status = ($? & 127);
                   }
                   else {
                       $status = $? >> 8;
                   }
	print "EXITSTATUS=$status\n";
	return $status;
    }
    
    print "Got userDN=$userDN\n";
    print "$now_string: Running: $cmd\n";
    if ( $configValues{gridUser} eq "" )
    {
	`chown $hash{user}:root $dgas_dir/$_[0].proxy`; #
    }
    else
    {
	print "$now_string: changing ownership of $_[0].proxy, to user: $configValues{gridUser}\n";
	`chown $configValues{gridUser}:root $dgas_dir/$_[0].proxy`;
    }
    `chmod 600 $dgas_dir/$_[0].proxy`;
    $ENV{X509_USER_PROXY} = "$dgas_dir/$_[0].proxy";
    if ( $configValues{gridUser} eq "" )
    {
    	$status = system("su  $hash{user}  -c '$cmd'");
    }
    else
    {
	print "$now_string: Executing command as user:$configValues{gridUser}\n";
	$status = system("su $configValues{gridUser} -c '$cmd'");
    }
    if ($? == -1) {
                     print "failed to execute: $!\n";
                  }
                  elsif ($? & 127) {
                      $status = ($? & 127);
                  }
                  else {
                      $status = $? >> 8;
                  }
    print "$_[0]:";
    print "EXITSTATUS=$status\n";
    delete $ENV{X509_USER_PROXY};
    return $status;
}


sub checkProxy()
{
	my $proxyFile = $_[0];
	my $userDN = $_[1];
#	my $user = $_[1];

	my $openssl = `which openssl`;
	if ( -e $openssl )
	{
		print "Found $openssl\n";	
	}
	else
	{
		$openssl = "/usr/bin/openssl";
		print "Trying with $openssl\n";
	}
	my $exitStatus = 0;
	if ( -e $openssl )
	{
		my $opensslCmd = "$openssl x509 -in $proxyFile -noout -checkend 20";
		$exitStatus = system($opensslCmd);
	}
	else
	{
		print "Warning couldn't find $openssl command\n";
	}
	$_ = `$openssl x509 -in $proxyFile -noout -subject`;
        s/(\/CN=(limited |)proxy)//gc;
        if ( /^subject= (.*)$/)
        {
	    my $buff = $1;
	    $buff =~ s/emailAddress/Email/g;
	    print "Substitued email Address field: $buff\n"; 
            $_[1] = $buff;
        }
	print "Openssl found user DN:$_[1]\n";
	return $exitStatus;
}

sub findVomsProxyInfo()
{
        my $buff = `which voms-proxy-info`;
        $buff =~ s/\n//g;
        if ( -e $buff )
        {
                $_[0] = $buff;
                return 0;
        }
        else
        {
                $buff = "/opt/glite/bin/voms-proxy-info";
                if ( -e $buff )
                {
                        $_[0] = $buff;
                        return 0;
                }
        }
        return 1;
}

sub findUserFqan()
{
        if ( -e $vomsProxyInfo )
        {
                my $fqan = `$vomsProxyInfo -file $_[0] -fqan`;
                $fqan =~ s/\n//g;
                $_[1] = $fqan;
                return 0;
        }
        return 1;
}



sub setFile()
{
    my @set = @_;   ## set[0] = fileName, set[1] = multiplicity, set[2] = depth
    
    if ($set[2] >= $qDept) 
    {
	$set[1]++;
	$set[2] = 0;
    }
    else {$set[2]++;}
    
    my $point = ":";
    my $updated = $set[0].$point.$set[1].$point.$set[2];
    
    return $updated;
}



sub delFile {   #receive as argument the filename to delete

    my $cmd = "rm -f $dgas_dir/$_[0]";
    print "$cmd \n";
    my $status = system($cmd);
    $cmd .=".proxy";
    print "$cmd \n";
    $status =system($cmd);
}



sub printOut {

    print "queue: $hash{queue} \n";      
    print "server: $hash{server} \n";      
    print "HLR_LOCATION: $hash{hlrLoc} \n";      
    print "EDG_WL_JOBID: $hash{jobId} \n";      
    print "cput: $hash{cput} \n";
    print "mem: $hash{mem} \n";
    print "vmem: $hash{vmem} \n";
    print "walltime: $hash{walltime} \n";  
    print "------------------------------------ \n";
}



sub timeOut
{
    my $stop = time;
    my $delta = $stop -$start;
    #print "delta = $delta \n" ;   
    
    if($delta > $T2)
    {
	$start= time;
	return 0;
    }
    
    return 1;
}


sub createErrDir
{
    my $cmd = "mkdir $dgas_err_dir";
    print "$cmd \n";
    my $status = system($cmd);

    return $status;
}



sub moveInErrDir
{
    my $cmd = "mv $dgas_dir/$_[0] $dgas_err_dir";
    my $status = system($cmd);
    print "$cmd \n";
    $cmd = "mv $dgas_dir/$_[0].proxy $dgas_err_dir";
    $status = system($cmd);
    print "$cmd \n";

    return $status;
}



##-------> lock  subroutines <---------##

sub putLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") && return 1;
    close(IN);

    open(OUT, "> $lockName") || return 2;
    print OUT  $$;    ## writes pid   
    close(OUT);
    return 0;
}

sub delLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") || return 1;
    close(IN);
    my $status = system("rm -f $lockName");
    return $status;
}


##-------> sig handlers subroutines <---------##

sub sigHUP_handler {
        print "got SIGHUP\n";
	$keepGoing = 1;
	goto START;
}

sub sigINT_handler {
        print "got SIGINT\n";
        $keepGoing = 0;
}
