#!/usr/bin/perl -w

#Gianduia, DGAS job monitor and Usage Records collector.
use strict;
use POSIX;
use Text::ParseWords;

my $sigset = POSIX::SigSet ->new();
my $actionHUP=POSIX::SigAction->new("sigHUP_handler",$sigset,&POSIX::SA_NODEFER);
my $actionInt=POSIX::SigAction->new("sigINT_handler",$sigset,&POSIX::SA_NODEFER);
POSIX::sigaction(&POSIX::SIGHUP, $actionHUP);
POSIX::sigaction(&POSIX::SIGINT, $actionInt);
POSIX::sigaction(&POSIX::SIGTERM, $actionInt);
my $configFilePath ="/opt/glite/etc/dgas_gianduia.conf"; 
my $lastId = "";
my %configValues = (
		    lsfAcctLogDir     =>  "",
		    lockFileName      => "/opt/glite/var/dgasOOB.lock",
		    bufferFileName      => "/opt/glite/var/dgasOOBuffer",
		    mainPollInterval  => "5",
		    queuePollInterval =>"25",
		    timeInterval      =>"5",
		    jobPerTimeInterval =>"10",
		    );

my %hash = (
             server   =>  "server",
             queue    =>  "queue",
             lrmsType =>  "lsf",
             globusResContact =>  "GLOBUS_RES_CONTACT",
             cput     =>  "cput",
             hlrLoc   =>  "HLR_LOCATION",
             jobId    =>  "WMS_JOBID",
             jobIdEdg =>  "EDG_WL_JOBID",
             user     =>  "PBS_O_LOGNAME",
             mem      =>  "mem",
             vmem     =>  "vmem",
             walltime =>  "walltime",
             lrmsId   =>  "lrmsId",           
             jobName  =>  "jobName",
             group    =>  "group", 
             ctime    =>  "ctime",
             qtime    =>  "qtime",
             etime    =>  "etime",
             start    =>  "start", 
             end      =>  "end",
             exitStatus      =>  "exitStatus", 
             execHost    =>  "execHost",
             processors => "processors",
             si2k => "GlueHostBenchmarkSI00",
             sf2k => "GlueHostBenchmarkSF00",
             tz => "timeZone",
             userVO => "",
             );

#Parse configuration file
my $startJob = "";
my $lastJob = "";
if(exists $ARGV[0]) 
{
    $configFilePath = $ARGV[0];
}
if(exists $ARGV[1])
{
    #start analising from $lastId and go backward instead
    #of starting from end of file.
    $lastId = $ARGV[1];	
    if(exists $ARGV[2])
    {
	$startJob = $ARGV[2];
    }
}
&parseConf($configFilePath); 

## Global variable initialization
my $UR;
my $list;

my $lsfAcctLogDir    = $configValues{lsfAcctLogDir};
my $timeInterval     = $configValues{timeInterval};
my $jobPerTimeInterval     = $configValues{jobPerTimeInterval};

my $lockFileName = $configValues{lockFileName};

my $timeZone = `date +%z`;
chomp($timeZone);
my $domainName = `hostname -d`;
chomp($domainName);
my $keepGoing = 1;
my $start1 = time;   ##  time init
my $start2 = time;   ##  time init
my @garbageQueue = ();

#put lock
if ( &putLock($lockFileName) != 0 )
{
    die "Fatal Error: Couldn't open lock file! in $lockFileName\n";
}
else
{
	print "Lock file succesfully created.\n";
}

#This script must be exexuted in the cron.
if ( $startJob eq "" )
{
	&readBuffer($configValues{bufferFileName}, $startJob);
}
print "First job analysed: $startJob\n";
&checkLsfLog($startJob,$lastJob);    
print "Last job analysed: $lastJob\n";
if ($lastJob ne "" )
{
	&putBuffer($configValues{bufferFileName}, $lastJob);
}


print "Exiting...\n";
if ( &delLock($lockFileName) != 0 )
{	
    print "Error removing lock file.\n";
}
else
{
    print "Lock file removed.\n";
}
print "Exit.\n";


###-----------------------END-----------------------------------------###

sub parseConf 
{
    my $fconf = $_[0];
    open(FILE, "$fconf") || die "file not open \n";
    while(<FILE>)
    {
	if(/^lsfAcctLogDir\s*=\s*\"(.*)\"$/){$configValues{lsfAcctLogDir}=$1;}
        if(/^oobLockFileName\s*=\s*\"(.*)\"$/){$configValues{lockFileName}=$1;}
        if(/^bufferFileName\s*=\s*\"(.*)\"$/){$configValues{bufferFileName}=$1;}
	if(/^mainPollInterval\s*=\s*\"(.*)\"$/){$configValues{mainPollInterval}=$1;}
	if(/^queuePollInterval\s*=\s*\"(.*)\"$/){$configValues{queuePollInterval}=$1;}
    }
    close(FILE);
}

sub checkLsfLog#FIXME
{
        my $startJob = $_[0];
	
        $lsfAcctLogDir= $configValues{lsfAcctLogDir};
        open ( LOGFILE, "/usr/bin/tac $lsfAcctLogDir/lsb.acct |" ) or return 1;
	if ( $lastId eq "" )
	{
		print "Parsing file from end to $startJob in $lsfAcctLogDir/lsb.acct\n";
	}
	else
	{
		 print "Parsing file from $lastId to $startJob in $lsfAcctLogDir/lsb.acct\n";
	}
	my $firstJobId = 1;
        while ( <LOGFILE> )
        {
		if ( $keepGoing == 0 )
		{
			print "Exiting from LSF loop...\n";
			close LOGFILE;
			return 1;
		}
                my @ARRAY = split(" " , $_ );
                my $lsfVersion = $ARRAY[1];
                $lsfVersion =~ s/\"//g;
                if ( $lsfVersion eq "4.1" ||
                        $lsfVersion eq "5.0" ||
                        $lsfVersion eq "5.1" ||
                        $lsfVersion eq "6.0" ||
                        $lsfVersion eq "6.1" )
                {
                        my $currentJobId = $ARRAY[3];
			if ( ($currentJobId ne $lastId) && ($lastId ne "") )
			{
				next;
			}
			else
			{
				$lastId = ""; ##so next check shall fail.
			}
                        if ( ( $startJob eq $currentJobId ) && ($startJob ne "") )
                        {
                                #Got the right line!
                                print "Found $startJob in LOG!\n";
				close LOGFILE;
                                return 0;
                        }
			else
			{
				#this record should be accounted
				if ( $firstJobId == 1 )
				{
					$_[1] = $currentJobId;
					$firstJobId = 0;
				}
				print "$currentJobId:";
				&parseUR_LSF($_);
				print "processing $hash{jobName}\n";
				&callAtmClient();
			}
                }
        }
        close LOGFILE;
        return 1;
}

sub parseUR_LSF
{
        my $URString = $_[0];
        my @new = quotewords(" ", 0, $URString);
        my $shift1 = $new[22];
        my $shift2 = $new[23+$shift1];
        $hash{server}=$new[16].".".$domainName;
        $hash{queue}=$new[12];
        $hash{user}=$new[11];
        $hash{lrmsId}=$new[3];
        $hash{processors}=$new[6];
        $hash{walltime}=$new[2]-$new[10];
        $hash{cput}=int($new[28+$shift2])+int($new[29+$shift2]);
        $hash{mem}=$new[54+$shift2]."k";
        $hash{vmem}=$new[55+$shift2]."k";
        $hash{start}=$new[10];
        $hash{end}=$new[2];
        $hash{jobName}=$new[26+$shift2];
        $hash{exitStatus}=$new[49+$shift2];
	&detectVOfromuid($hash{user});
}

sub detectVOfromuid
{
	$_=$_[0];
	if(/(.*)\d\d\d/)
	{
		$hash{userVO}=$1;
	}
	else
	{
		$hash{userVO}="";
	}
	
}

sub callAtmClient 
{
    my $now_string = localtime;
    if ( $keepGoing == 0 )
    {
        print "$now_string: Got termination signal..\n";
        return -2;
    }
    my $cmd = "";
    my $status = -1; 
    my $resGridId="$hash{server}:$hash{queue}";
    $cmd = "/opt/glite/libexec/glite_dgas_atmClient";
    $cmd .= " --jobid $hash{server}:$hash{lrmsId}_$hash{start}";
    $cmd .= " --resgridid $resGridId";
    $cmd .= " CPU_TIME=$hash{cput} WALL_TIME=$hash{walltime} PMEM=$hash{mem} VMEM=$hash{vmem}";
    $cmd .= " \"QUEUE=$hash{queue}\" \"USER=$hash{user}\" \"LRMSID=$hash{lrmsId}\"";
    $cmd .= " \"PROCESSORS=$hash{processors}\" \"URCREATION=$now_string\"";
    $cmd .= " \"group=$hash{group}\" \"jobName=$hash{jobName}\"";
    $cmd .= " \"start=$hash{start}\" \"end=$hash{end}\"";
    $cmd .= " \"ctime=$hash{ctime}\" \"qtime=$hash{qtime}\"";
    $cmd .= " \"etime=$hash{etime}\" \"exitStatus=$hash{exitStatus}\"";
    $cmd .= " \"si2k=$hash{si2k}\" \"sf2k=$hash{sf2k}\"";
    $cmd .= " \"tz=$timeZone\" \"accountingProcedure=outOfBand\"";
    $cmd .= " \"fqan=/$hash{userVO}/Role=NULL/Capability=NULL\""; #!!!!!!
    $cmd .= " --usrcert \"$hash{user}:$hash{server}\"";
    $cmd .= " -3";
    $status = system("$cmd");
    print "$now_string: Running: $cmd\n";
    if ($? == -1) {
           print "failed to execute: $!\n";
    }
    elsif ($? & 127) {
           $status = ($? & 127);
    }
    else {
           $status = $? >> 8;
    }
    print "EXITSTATUS=$status\n\n";
    return $status;
}


##-------> lock  subroutines <---------##

sub putLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") && return 1;
    close(IN);
    open(OUT, "> $lockName") || return 2;
    print OUT  $$;    ## writes pid   
    close(OUT);
    return 0;
}

sub delLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") || return 1;
    close(IN);
    my $status = system("rm -f $lockName");
    return $status;
}

##-------> sig handlers subroutines <---------##

sub putBuffer
{
    my $buffName = $_[0];
    if ($keepGoing == 1) {
	# this is done only if no SIGINT was received!
	open(OUT, "> $buffName") || return 2;
	print OUT  $_[1];    ## writes lastId
	close(OUT);
    }
    return 0;
}

sub readBuffer
{
	my $buffname = $_[0];
	open(IN, "< $buffname") || return 2;
	while ( <IN> )
	{
		$_[1] = $_;
	}	
	close(IN);
	return 0;
}

sub sigHUP_handler {
        print "got SIGHUP\n";
	$keepGoing = 1;
}

sub sigINT_handler {
        print "got SIGINT\n";
        $keepGoing = 0;
}
