#!/usr/bin/perl -w

#Gianduia, DGAS job monitor and Usage Records collector.
use strict;
use POSIX;
use Text::ParseWords;

my $sigset = POSIX::SigSet ->new();
my $actionHUP=POSIX::SigAction->new("sigHUP_handler",$sigset,&POSIX::SA_NODEFER);
my $actionInt=POSIX::SigAction->new("sigINT_handler",$sigset,&POSIX::SA_NODEFER);
POSIX::sigaction(&POSIX::SIGHUP, $actionHUP);
POSIX::sigaction(&POSIX::SIGINT, $actionInt);
POSIX::sigaction(&POSIX::SIGTERM, $actionInt);
my $configFilePath ="/opt/glite/etc/dgas_gianduia.conf"; 
my %configValues = (
		    pbsAcctLogDir     =>  "",
		    lockFileName      => "/opt/glite/var/dgasOOB.lock",
		    bufferFileName      => "/opt/glite/var/dgasOOBuffer",
		    mainPollInterval  => "5",
		    queuePollInterval =>"25",
		    timeInterval      =>"5",
		    jobPerTimeInterval =>"10",
		    );

my %hash = (
             server   =>  "server",
             queue    =>  "queue",
             lrmsType =>  "pbs",
             globusResContact =>  "GLOBUS_RES_CONTACT",
             cput     =>  "cput",
             hlrLoc   =>  "HLR_LOCATION",
             jobId    =>  "WMS_JOBID",
             jobIdEdg =>  "EDG_WL_JOBID",
             user     =>  "PBS_O_LOGNAME",
             mem      =>  "mem",
             vmem     =>  "vmem",
             walltime =>  "walltime",
             lrmsId   =>  "lrmsId",           
             jobName  =>  "jobName",
             group    =>  "group", 
             ctime    =>  "ctime",
             qtime    =>  "qtime",
             etime    =>  "etime",
             start    =>  "start", 
             end      =>  "end",
             exitStatus      =>  "exitStatus", 
             execHost    =>  "execHost",
             processors => "processors",
             si2k => "GlueHostBenchmarkSI00",
             sf2k => "GlueHostBenchmarkSF00",
             tz => "timeZone",
             userVO => "",
             );

#Parse configuration file
if(exists $ARGV[0]) 
{
    $configFilePath = $ARGV[0];
}	
&parseConf($configFilePath); 

## Global variable initialization
my $UR;
my $list;

my $pbsAcctLogDir    = $configValues{pbsAcctLogDir};
my $timeInterval     = $configValues{timeInterval};
my $jobPerTimeInterval     = $configValues{jobPerTimeInterval};

my $lockFileName = $configValues{lockFileName};

my $T2 = $configValues{queuePollInterval};
my $T1 = $configValues{mainPollInterval};
my $garbageCleanInterval = 10*$T2;

my $timeZone = `date +%z`;
chomp($timeZone);
my $domainName = `hostname -d`;
chomp($domainName);
my $keepGoing = 1;
my $start1 = time;   ##  time init
my $start2 = time;   ##  time init
my @garbageQueue = ();

#put lock
if ( &putLock($lockFileName) != 0 )
{
    die "Fatal Error: Couldn't open lock file! in $lockFileName\n";
}
else
{
	print "Lock file succesfully created.\n";
}

#This script must be exexuted in the cron.
my $startJob = "";
my $startFile = "";
my $lastJob = "";
my $lastFile = "";
&readBuffer($configValues{bufferFileName}, $startJob, $startFile);
print "First job to analyse: $startFile:$startJob\n";
&checkPbsLog($startJob,$startFile,$lastFile,$lastJob);    
print "Last job analysed: $lastFile:$lastJob\n";
if ($lastJob ne "" )
{
	&putBuffer($configValues{bufferFileName}, $lastJob, $lastFile);
}


print "Exiting...\n";
if ( &delLock($lockFileName) != 0 )
{	
    print "Error removing lock file.\n";
}
else
{
    print "Lock file removed.\n";
}
print "Exit.\n";


###-----------------------END-----------------------------------------###

sub parseConf 
{
    my $fconf = $_[0];
    open(FILE, "$fconf") || die "file not open \n";
    while(<FILE>)
    {
	if(/^pbsAcctLogDir\s*=\s*\"(.*)\"$/){$configValues{pbsAcctLogDir}=$1;}
	if(/^oobLockFileName\s*=\s*\"(.*)\"$/){$configValues{lockFileName}=$1;}
	if(/^bufferFileName\s*=\s*\"(.*)\"$/){$configValues{bufferFileName}=$1;}
	if(/^mainPollInterval\s*=\s*\"(.*)\"$/){$configValues{mainPollInterval}=$1;}
	if(/^queuePollInterval\s*=\s*\"(.*)\"$/){$configValues{queuePollInterval}=$1;}
    }
    close(FILE);
}

sub checkPbsLog#FIXME
{
        my $startJob = $_[0]; #jobNumber to start with.
        my $startFile = $_[1]; #jobNumber to start with.
	my $lastFile= $_[2]; #last file analysed on previus run
        $pbsAcctLogDir= $configValues{pbsAcctLogDir};
	my @dirs;
	opendir(DIR,$pbsAcctLogDir ) || die "can't opendir $pbsAcctLogDir: $!";
	while( defined(my $file = readdir(DIR)) )
	{
		push @dirs, $file;
	}
	closedir DIR;
	my $goOn = 1;
	my $firstJobId = 1;
	my $firstFileOpened = 0;
	print "Scanning logdir $pbsAcctLogDir until file:$startFile\n";
	while ( $goOn && $keepGoing && @dirs && !$firstFileOpened )
	{
		my $file = pop @dirs; #going backward
		next if (($file eq ".") || ($file eq ".."));
		if ( $file eq $startFile )
		{
			##this file has already been fully processed
			##so we can break here.
			#print "Found $startFile in dir no need to go further!\n";
			#return 0;
			print "Found $startFile in dir; last file to process!\n";
			$firstFileOpened = 1;
		}
		open ( LOGFILE, "/usr/bin/tac $pbsAcctLogDir/$file |" ) or return 1;
		print "Parsing file from end to $startJob in $pbsAcctLogDir/$file\n";
		while ( <LOGFILE> )
		{
			if ( $keepGoing == 0 )
			{
				print "Exiting from PBS loop...\n";
				close LOGFILE;
				return 1;
			}
			my @ARRAY = split(" " , $_ );
			my @ARRAY2 = split(";" , $ARRAY[1] );
			my $targetJobId = $ARRAY2[2];
			my $lineType = $ARRAY2[1];
			if ( $lineType ne "E" )
			{
				next;
			}
			if ( ( $startJob eq $targetJobId ) && ($startJob ne "") )
			{
				#Got the right line!
				print "Found $targetJobId in LOG!\n";
				close LOGFILE;
				return 0;
			}	
			else
			{
				if ( $firstJobId == 1 )
				{
					$_[2] = $file;
					$_[3] = $targetJobId;
					$firstJobId = 0;
				}
				print "$targetJobId:";
				&parseUR_PBS($_);
				print "processing $hash{jobName}\n";
				&callAtmClient();
			}
		}
	}
}

sub parseUR_PBS
{
	my $URString = $_[0];
	my @URArray = split ( ' ' , $URString );
	my @tmpArray = split ( ';', $URArray[1] );
	$_ = $tmpArray[3];
	if (/^user=(.*)$/){$hash{user}=$1;};
	$hash{lrmsId}=$tmpArray[2];
	$_ = $tmpArray[2];
	if (/^(\d*)\.(.*)$/){$hash{server}=$2;};
	foreach ( @URArray )
	{
		if ( /^queue=(.*)$/){$hash{queue}=$1;};
		if ( /^resources_used.cput=(.*)$/)
                {
                        $_=$1;
                        $_=~ /(\d\d):(\d\d):(\d\d)$/;
                        $hash{cput}= $3 + $2*60 + $1*3600;
                }
                if ( /^resources_used.walltime=(.*)$/)
                {
                        $_=$1;
                        $_=~ /(\d\d):(\d\d):(\d\d)$/;
                        $hash{walltime}= $3 + $2*60 + $1*3600;
                }
                if ( /^resources_used.vmem=(.*)$/)
                {
                        $_=$1;
                        $_=~ /(\d*[M.k]b)$/;
                        $hash{vmem}     = $1;
                }
                if ( /^resources_used.mem=(.*)$/)
                {
                        $_=$1;
                        $_=~ /(\d*[M.k]b)$/;
                        $hash{mem}     = $1;
                }
                if ( /^Resource_List.neednodes=(.*)$/)
                {
                        $hash{processors} = $1;
                }
		if ( /^group=(.*)$/)
                {
                        $hash{group} = $1;
                }
                if ( /^jobname=(.*)$/)
                {
                        $hash{jobName} = $1;
                }
                if ( /^ctime=(.*)$/)
                {
                        $hash{ctime} = $1;
                }
                if ( /^qtime=(.*)$/)
                {
                        $hash{qtime} = $1;
                }
                if ( /^etime=(.*)$/)
                {
                        $hash{etime} = $1;
                }
                if ( /^start=(.*)$/)
                {
                        $hash{start} = $1;
                }
                if ( /^end=(.*)$/)
                {
                        $hash{end} = $1;
                }
                if ( /^exec_host=(.*)$/)
                {
                        $hash{execHost} = $1;
                }
                if ( /^Exit_status=(.*)$/)
                {
                        $hash{exitStatus} = $1;
                }
	}
	&detectVOfromuid($hash{user});
}

sub detectVOfromuid
{
	$_=$_[0];
	if(/(.*)\d\d\d/)
        {
                $hash{userVO}=$1;
        }
        else
        {
                $hash{userVO}="";
        }
}

sub callAtmClient 
{
    my $now_string = localtime;
    if ( $keepGoing == 0 )
    {
        print "$now_string: Got termination signal..\n";
        return -2;
    }
    my $cmd = "";
    my $status = -1; 
    my $resGridId="$hash{server}:$hash{queue}";
    $cmd = "/opt/glite/libexec/glite_dgas_atmClient";
    $cmd .= " --jobid $hash{server}:$hash{lrmsId}_$hash{start}";
    $cmd .= " --resgridid $resGridId";
    $cmd .= " CPU_TIME=$hash{cput} WALL_TIME=$hash{walltime} PMEM=$hash{mem} VMEM=$hash{vmem}";
    $cmd .= " \"QUEUE=$hash{queue}\" \"USER=$hash{user}\" \"LRMSID=$hash{lrmsId}\"";
    $cmd .= " \"PROCESSORS=$hash{processors}\" \"URCREATION=$now_string\"";
    $cmd .= " \"group=$hash{group}\" \"jobName=$hash{jobName}\"";
    $cmd .= " \"start=$hash{start}\" \"end=$hash{end}\"";
    $cmd .= " \"ctime=$hash{ctime}\" \"qtime=$hash{qtime}\"";
    $cmd .= " \"etime=$hash{etime}\" \"exitStatus=$hash{exitStatus}\"";
    $cmd .= " \"si2k=$hash{si2k}\" \"sf2k=$hash{sf2k}\"";
    $cmd .= " \"tz=$timeZone\" \"accountingProcedure=outOfBand\"";
    $cmd .= " \"fqan=/$hash{userVO}/Role=NULL/Capability=NULL\""; #!!!!!!
    $cmd .= " --usrcert \"$hash{user}:$hash{server}\"";
    $cmd .= " -3";
    $status = system("$cmd");
    print "$now_string: Running: $cmd\n";
    if ($? == -1) {
           print "failed to execute: $!\n";
    }
    elsif ($? & 127) {
           $status = ($? & 127);
    }
    else {
           $status = $? >> 8;
    }
    print "EXITSTATUS=$status\n\n";
    return $status;
}


##-------> lock  subroutines <---------##

sub putLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") && return 1;
    close(IN);
    open(OUT, "> $lockName") || return 2;
    print OUT  $$;    ## writes pid   
    close(OUT);
    return 0;
}

sub delLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") || return 1;
    close(IN);
    my $status = system("rm -f $lockName");
    return $status;
}

##-------> sig handlers subroutines <---------##

sub putBuffer
{
    my $buffName = $_[0];
    if ($keepGoing == 1) {
	# this is done only if no SIGINT was received!
	open(OUT, "> $buffName") || return 2;
	print OUT  "$_[1]:$_[2]";    ## writes lastId
	close(OUT);
    }
    return 0;
}

sub readBuffer
{
	my $buffname = $_[0];
	open(IN, "< $buffname") || return 2;
	my $line;
	my $file;
	while ( <IN> )
	{
		($line,$file) = split(':');
		$_[1] = $line;
		$_[2] = $file;
	}	
	close(IN);
	return 0;
}

sub sigHUP_handler {
        print "got SIGHUP\n";
	$keepGoing = 1;
}

sub sigINT_handler {
        print "got SIGINT\n";
        $keepGoing = 0;
}
