#!/usr/bin/perl -w

#Gianduia, DGAS job monitor and Usage Records collector.
use strict;
use POSIX;

my $sigset = POSIX::SigSet ->new();
my $actionHUP=POSIX::SigAction->new("sigHUP_handler",$sigset,&POSIX::SA_NODEFER);
my $actionInt=POSIX::SigAction->new("sigINT_handler",$sigset,&POSIX::SA_NODEFER);
POSIX::sigaction(&POSIX::SIGHUP, $actionHUP);
POSIX::sigaction(&POSIX::SIGINT, $actionInt);
POSIX::sigaction(&POSIX::SIGTERM, $actionInt);
my $configFilePath ="/opt/glite/etc/dgas_gianduia.conf"; 
my %configValues = (
		    chocolateBox      =>  "/opt/glite/var/dgasRawbox",
		    gianduiottiBox    =>  "/opt/glite/var/dgasURbox",
		    garbageCollector  =>  "/opt/edg/var/dgas_garbage",
		    pbsAcctLogDir     =>  "/var/spool/pbs/server_priv/accounting",
		    lsfAcctLogDir     =>  "",
		    lockFileName      => "/opt/glite/var/dgas_gianduia.lock",
		    mainPollInterval  => "5",
		    queuePollInterval =>"25",
		    timeInterval      =>"5",
		    jobPerTimeInterval =>"10",
		    );


#Parse configuration file
if(exists $ARGV[0]) 
{
    $configFilePath = $ARGV[0];
}	
&parseConf($configFilePath); 

## Global variable initialization
my %hash;
my $UR;
my $list;

my $chocolateBox     = $configValues{chocolateBox};
my $gianduiottiBox   = $configValues{gianduiottiBox};
my $garbageCollector = $configValues{garbageCollector};
my $pbsAcctLogDir    = $configValues{pbsAcctLogDir};
my $lsfAcctLogDir    = $configValues{lsfAcctLogDir};
my $timeInterval     = $configValues{timeInterval};
my $jobPerTimeInterval     = $configValues{jobPerTimeInterval};

my $lockFileName = $configValues{lockFileName};

my $T2 = $configValues{queuePollInterval};
my $T1 = $configValues{mainPollInterval};
my $garbageCleanInterval = 10*$T2;

my $keepGoing = 1;
my $start1 = time;   ##  time init
my $start2 = time;   ##  time init
my @garbageQueue = ();

( -d $configValues{chocolateBox} ) || mkdir $configValues{chocolateBox};
#chocaolate box should be public writable, since the jobWrapper run
# with user priviledges
chmod 0777, $configValues{chocolateBox};
( -d $configValues{gianduiottiBox} ) || mkdir $configValues{gianduiottiBox};
( -d $configValues{garbageCollector} ) || mkdir $configValues{garbageCollector};


#put lock
if ( &putLock($lockFileName) != 0 )
{
    die "Fatal Error: Couldn't open lock file! in $lockFileName\n";
}
else
{
	print "Lock file succesfully created.\n";
}


while( $keepGoing )
{
    
##----------------------------------------------------------------  STEP1
    my $j = 0;
    opendir(DIR, $chocolateBox) || die "can't opendir $chocolateBox: $!";
    while( defined(my $file = readdir(DIR)) && $keepGoing ) 
    {
	if ( $j == $jobPerTimeInterval )
	{
		$j = 0;
		print "$j jobs processed. Sleeping $timeInterval seconds.\n";
		sleep $timeInterval;
	}
	$j++;
	print "Processing file:$chocolateBox/$file\n";
	next if ($file =~ /^\.\.?$/ || $file =~ /.proxy$/ || $file =~ /^$garbageCollector$/);	# skip '.' '..' '.proxy' files
	next if ( !( ($file =~ "pbs_") || ($file =~ "lsf_")) );
	    if (&checkLog($file) == 0)
	    {   ## file is a true input file and is in the acct log
		## so it is a job that alraedy ended.
		print "Found $file\n";
		#mv the file to $gianduiottiBox
		if ( &moveToGianduiottiBox($file) != 0 )
		{
			print "Warning: Error moving $file to gianduiottiBox!\n";
		}
		else
		{
			print "Moved $file to gianduiottiBox!\n";
		}
	    }
	    else
	    {
		#&pushToQueue($file);
		print "Job not found in acct log file\n";
	    }
    }
    closedir DIR; 
    print "Sleeping for $T1 seconds\n";
    sleep($T1);

##----------------------------------------------------------------  END STEP1 
    
    if(( time-$start1 > $T2 ))    ##timeout reached
    {
	$start1=time;
	##check garbage 
	opendir(DIR, $chocolateBox) || die "can't opendir $chocolateBox: $!";
	while( defined(my $file = readdir(DIR)) )
	{
		next if ($file =~ /^\.\.?$/ || $file =~ /.proxy$/ || $file =~ /^$garbageCollector$/);
	#	&checkGarbage($file)
	}
	closedir DIR;
    }
    my $DELTA2 = time-$start2;
    if (( time-$start2 > $garbageCleanInterval ))
    {
	$start2 = time;
       #&garbageClean(); FIXME workaround to infinite loop, fix ASAP
    }
    
}   

print "Exiting...\n";
if ( &delLock($lockFileName) != 0 )
{	
    print "Error removing lock file.\n";
}
else
{
    print "Lock file removed.\n";
}
print "Exit.\n";


###-----------------------END-----------------------------------------###

sub parseConf 
{
    my $fconf = $_[0];
    open(FILE, "$fconf") || die "file not open \n";
    while(<FILE>)
    {
	if (/\$\{(.*)\}/)
        {
                my $value=$ENV{$1};
                s/\$\{$1\}/$value/g;
        }
	if(/^pbsAcctLogDir\s*=\s*\"(.*)\"$/){$configValues{pbsAcctLogDir}=$1;}
	if(/^lsfAcctLogDir\s*=\s*\"(.*)\"$/){$configValues{lsfAcctLogDir}=$1;}
	if(/^chocolateBox\s*=\s*\"(.*)\"$/){$configValues{chocolateBox}=$1;}
	if(/^gianduiottiBox\s*=\s*\"(.*)\"$/){$configValues{gianduiottiBox}=$1;}
	if(/^garbageCollector\s*=\s*\"(.*)\"$/){$configValues{garbageCollector}=$1;}
	if(/^lockFileName\s*=\s*\"(.*)\"$/){$configValues{lockFileName}=$1;}
	if(/^mainPollInterval\s*=\s*\"(.*)\"$/){$configValues{mainPollInterval}=$1;}
	if(/^queuePollInterval\s*=\s*\"(.*)\"$/){$configValues{queuePollInterval}=$1;}
    }
    close(FILE);
}

sub checkPbsLog
{
	my $pbsJobName;
	$_=$_[0];
	if(/^pbs_(.*)$/){$pbsJobName=$1;}
	print "Checking pbs log for job:$pbsJobName\n";
	my @dirs;
	opendir(DIR,$pbsAcctLogDir ) || die "can't opendir $pbsAcctLogDir: $!";
        while( defined(my $file = readdir(DIR)) )
	{
		push @dirs, $file;
	}
	closedir DIR;
	my $goOn = 1;
	my $output = "";
	while ( $goOn && $keepGoing && @dirs )
	{
		print "Pbs while loop";
		my $file = pop @dirs;
		next if (($file eq ".") || ($file eq ".."));
		my $cmd = "grep $pbsJobName $pbsAcctLogDir/$file | grep Exit_status";
		print "Executing:$cmd\n";
		$output = `$cmd`;
		print "Output=$output\n";
		if ( $output ne "" )
		{
			print "Exiting from pbs grep loop";
			$goOn = 0;
		}
	}
	if ( $output eq "" )
	{
		print "Could not find info.\n";
		return 1;
	}
	else
	{
		print "Found job Info.\n";
		$_[1]=$output;
		return 0;
	}
}

sub findLsfLog
{
        #we are in LSF need to know the acct log dir
        my $lsfAcctLog;
        my $lsfConfFile = "$ENV{LSF_ENVDIR}/lsf.conf";
        my $lsfBinDir = "";
        my $lsfShareDir = "";
        my $clusterName = "";
        if ( -e $lsfConfFile )
        {
                my $lsfBinDirBuff = `grep LSF_BINDIR $lsfConfFile`;
                if ( $lsfBinDirBuff =~ /^LSF_BINDIR=(.*)$/)
                {
                        $lsfBinDir = $1;
                }
                my $lsfShareDirBuff = `grep LSB_SHAREDIR $lsfConfFile`;
                if ( $lsfShareDirBuff =~ /^LSB_SHAREDIR=(.*)$/)
                {
                        $lsfShareDir = $1;
                }
                $clusterName = `lsid | grep cluster`;
                if ( $clusterName =~ /My cluster name is (.*)$/ )
                {
                        $clusterName = $1;
                }
        }
        $lsfAcctLog="$lsfShareDir/$clusterName/logdir";
        if ( -d $lsfAcctLog )
        {
                return $lsfAcctLog;
        }
        else
        {
                return "";
        }
}

sub checkLsfLog#FIXME
{
	$_=$_[0];
        print "LSFFILE:$_[0]";
        my $lsfJobName = "";
        if(/^lsf_(.*)$/){$lsfJobName=$1;}
        my $logDirBuff = &findLsfLog();
        if ( $logDirBuff ne "" )
        {
                #we found the LSF log dir
                # so we set $lsfAcctLogDir accordingly
                #otherwise we use the one coming from teh configuration file
                $lsfAcctLogDir=$logDirBuff;
        }
        open ( LOGFILE, "/usr/bin/tac $lsfAcctLogDir/lsb.acct |" ) or return 1;
	print "Looking $lsfJobName in $lsfAcctLogDir/lsb.acct\n";
        while ( <LOGFILE> )
        {
		if ( $keepGoing == 0 )
		{
			print "Exiting from LSF loop...\n";
			close LOGFILE;
			return 1;
		}
                my @ARRAY = split(" " , $_ );
                my $lsfVersion = $ARRAY[1];
                $lsfVersion =~ s/\"//g;
                if ( $lsfVersion eq "4.1" ||
                        $lsfVersion eq "5.0" ||
                        $lsfVersion eq "5.1" ||
                        $lsfVersion eq "6.0" ||
                        $lsfVersion eq "6.1" )
                {
                        my $jobID = $ARRAY[3];
                        my $targetJobId = $jobID;
                        if ( $lsfJobName eq $targetJobId )
                        {
                                #Got the right line!
                                print "Found $lsfJobName in LOG!\n";
                                $_[1] = $_;
				close LOGFILE;
                                return 0;
                        }
                }
        }
        close LOGFILE;
        return 1;
}

sub isFileEmpty
{
        my $file = "$_[0]";
        my $fileContent = `cat $file`;
        if ( $fileContent eq "" )
        {
                return 0;
        }
        else
        {
                return 1;
        }
}

sub checkLog
{
   my $file= "$_[0]";
   my $now_string = localtime;
   if ( &isFileEmpty("$chocolateBox/$file") == 0 )
   {
        print "The file $file is empty, will retry...\n";
        return 1;
   }
   my $output;
   if ( $file =~ "pbs_" )
   {
	print "$now_string:Checking PBS acct log\n";
	if ( &checkPbsLog($file,$output) != 0 )
	{
		return 1;
	}
	else
	{
		#Append the collected info to $file
		$output = "ACCTLOG:$output\n";
		my $cmd = "echo \'$output\' >> $chocolateBox/$file";
		return system($cmd);
	}
   }
   if ( $file =~ "lsf_" )
   {
	my $retryCount = 0;
	my $maxRetryCount = 3;
	while ( ($retryCount < $maxRetryCount) && $keepGoing )
	{
		print "$now_string:Checking LSF acct log\n";
		if ( &checkLsfLog($file,$output) != 0 )
		{
			if ( $retryCount < $maxRetryCount )
			{	
				$retryCount++;
				print "$retryCount: retrying $file\n";
				sleep 1;
			}
			else
			{
				print "MaxRetryCount for LSF reached.\n";
				return 1;
			}
		}
		else
		{
			#Append the collected info to $file
			$output = "ACCTLOG:$output\n";
			my $cmd = "echo \'$output\' >> $chocolateBox/$file";
			return system($cmd);
		}
	}
   }
   return 2;
}

sub checkGarbage
{
	my $file = "$_[0]";
		my $pbsJobName;
		my $lsfJobName;
		$_=$file;
		if(/^pbs_(.*)$/)
		{
			sleep 10;
			$pbsJobName=$1;
			my $cmd = "qstat $pbsJobName";
			my $status = system($cmd);
			if ( $status != 0 )
			{
				#Ghost job
				&pushToGarbageQueue($file);
				&pushToGarbageQueue("$file.proxy");
				return 0;
			}
		}
		elsif(/^lsf_(.*)$/)
		{
			sleep 10;
			$lsfJobName=$1;
			print $lsfJobName;
			my $cmd = "bjobs $lsfJobName";
			print "$cmd\n";
			my $status = system($cmd);
			if ( $status != 0 )
			{
				#Ghost job
				&pushToGarbageQueue($file);
				&pushToGarbageQueue("$file.proxy");
				return 0;
			}
		}
		else
		{
			##real garbage
			&moveToGarbage($file);
			return 0;
		}
}

sub garbageClean
{
	print @garbageQueue;
	foreach (@garbageQueue)
	{
		my $file = $_;
		if ( $file =~ "pbs_")
		{
			next if ( &checkPbsLog($file) == 0);
		}
		if ( $file =~ "lsf_" )
		{
			next if ( &checkLsfLog($file) == 0);
		}
		my $cmd = "mv $chocolateBox/$file $garbageCollector/";
		if ( system($cmd) != 0 )
		{
			print "Warning: Error deleting file: $file\n";
			next;
		}
	}
	@garbageQueue = ();
}

sub pushToGarbageQueue
{
	my $file = $_[0];
	if ( &checkQueue($file) != 0 )
	{
		unshift @garbageQueue, $file;
	}
}

sub checkQueue
{
	foreach (@garbageQueue)
	{
		my $file = $_;
		if($file =~ /($_[0])/){return 0;}
	}
	return 1;
}

sub mvFile {   #receive as argument the filename to delete

    my $cmd = "mv $chocolateBox/$_[0] $gianduiottiBox/$_[0]";
    my $status = system($cmd);
    $cmd = "mv $chocolateBox/$_[0].proxy $gianduiottiBox/$_[0].proxy";
    $status =system($cmd);
}

sub moveToGarbage
{
    my $cmd = "mv $chocolateBox/$_[0] $garbageCollector/";
    my $status = system($cmd);
    if ( $status != 0 )
    {
	print "Warning error moving $_[0] to garbage collector\n";
    }
    return $status;
}

sub moveToGianduiottiBox
{
   my $cmd = "mv $chocolateBox/$_[0] $gianduiottiBox/";
   my $status = system($cmd);
   $cmd = "mv $chocolateBox/$_[0].proxy $gianduiottiBox/";
   $status += system($cmd);
   return $status;
}

sub untarFile
{
   my $file = $_[0];
   if($file =~ /.tar$/)
   {
	##it is a tar file, deflating it
	my $cmd = "tar xv --directory=$chocolateBox/  -f $file";
	my $res = system($cmd);
	if ( $res != 0 )
	{
		##There was an error processing the tar file
		print "Error unpacking $file\n";
	}
	else
	{
		$cmd = "rm -f $file";
		return system($cmd);
	}
   }
   else
   {
	##This is not a TAR file
	return 1;
   }
   
}

##-------> lock  subroutines <---------##

sub putLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") && return 1;
    close(IN);
    open(OUT, "> $lockName") || return 2;
    print OUT  $$;    ## writes pid   
    close(OUT);
    return 0;
}

sub delLock
{
    my $lockName = $_[0];
    open(IN,  "< $lockName") || return 1;
    close(IN);
    my $status = system("rm -f $lockName");
    return $status;
}

##-------> sig handlers subroutines <---------##

sub sigHUP_handler {
        print "got SIGHUP\n";
	$keepGoing = 1;
}

sub sigINT_handler {
        print "got SIGINT\n";
        $keepGoing = 0;
}
